// Copyright (c) 1996-2002 Brian D. Carlstrom

package bdc.scheme.expression;

import bdc.scheme.Environment;
import bdc.scheme.Scheme;
import bdc.scheme.SchemeException;
import bdc.scheme.Stack;
import bdc.scheme.Writer;
import bdc.scheme.compiler.CompileTimeEnvironment;
import bdc.util.Fmt;

public class LexicalAssignment extends Expression
{
    LexicalAddress lexicalAddress;
    Expression     expression;

    public LexicalAssignment (LexicalAddress lexicalAddress,
                              Expression     expression)
    {
        this.lexicalAddress = lexicalAddress;
        this.expression     = expression;
    }

    public Object eval (Environment environment, Stack stack)
      throws SchemeException
    {
        try {
            environment.lexicalAddressSet(lexicalAddress,
                                          expression.eval(environment, stack));
            return Scheme.Unspecified;
        }
        catch (SchemeException se) {
            backTrace(se);
            return Scheme.NotReached;
        }
    }

    /**
        Fixup sub-expressions and return

        lexicalAddress can't change so we don't have to cast and reset it
    */
    public Expression fixupVariables (CompileTimeEnvironment environment)
    {
        lexicalAddress.fixupVariables(environment);
        expression = expression.fixupVariables(environment);
        return this;
    }

    public String toString ()
    {
        return
            Fmt.S("(set! %s %s)",
                  lexicalAddress.variable.name,
                  Writer.write(expression));
    }
}
